@@ -26,6 +26,7 @@
 
 #include "idle-connection.h"
 #include "idle-handles.h" /* to check for valid nick */
+#include "idle-debug.h"
 
 G_DEFINE_TYPE(IdleConnectionManager, idle_connection_manager, TP_TYPE_BASE_CONNECTION_MANAGER)
 
@@ -62,18 +63,6 @@ static void _params_free(gpointer ptr) {
 	g_slice_free(Params, params);
 }
 
-enum {
-	PARAM_ACCOUNT,
-	PARAM_SERVER,
-	PARAM_PORT,
-	PARAM_PASSWORD,
-	PARAM_FULLNAME,
-	PARAM_CHARSET,
-	PARAM_QUIT_MESSAGE,
-	PARAM_USE_SSL,
-	LAST_PARAM
-};
-
 gboolean
 filter_nick(const TpCMParamSpec *paramspec, GValue *value, GError **error)
 {
@@ -93,7 +82,9 @@ static const TpCMParamSpec _params[] = {
 	{"account", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, TP_CONN_MGR_PARAM_FLAG_REQUIRED, NULL, G_STRUCT_OFFSET(Params, account), filter_nick},
 	{"server",  DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, TP_CONN_MGR_PARAM_FLAG_REQUIRED, NULL, G_STRUCT_OFFSET(Params, server)},
 	{"port", DBUS_TYPE_UINT16_AS_STRING, G_TYPE_UINT, TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GINT_TO_POINTER(6667), G_STRUCT_OFFSET(Params, port)},
-	{"password", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, 0, NULL, G_STRUCT_OFFSET(Params, password)},
+	{"password", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING,
+		TP_CONN_MGR_PARAM_FLAG_SECRET,
+		NULL, G_STRUCT_OFFSET(Params, password)},
 	{"fullname", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, 0, NULL, G_STRUCT_OFFSET(Params, fullname)},
 	{"username", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, 0, NULL, G_STRUCT_OFFSET(Params, username)},
 	{"charset", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, "UTF-8", G_STRUCT_OFFSET(Params, charset)},
@@ -112,12 +103,23 @@ static TpBaseConnection *_iface_new_connection(TpBaseConnectionManager *self, co
 static void idle_connection_manager_init(IdleConnectionManager *obj) {
 }
 
+static void
+idle_connection_manager_finalize (GObject *object)
+{
+	idle_debug_free ();
+
+	G_OBJECT_CLASS (idle_connection_manager_parent_class)->finalize (object);
+}
+
 static void idle_connection_manager_class_init(IdleConnectionManagerClass *klass) {
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	TpBaseConnectionManagerClass *parent_class = TP_BASE_CONNECTION_MANAGER_CLASS(klass);
 
 	parent_class->new_connection = _iface_new_connection;
 	parent_class->cm_dbus_name = "idle";
 	parent_class->protocol_params = _protocols;
+
+	object_class->finalize = idle_connection_manager_finalize;
 }
 
 static TpBaseConnection *_iface_new_connection(TpBaseConnectionManager *self, const gchar *proto, TpIntSet *params_present, void *parsed_params, GError **error) {
