@@ -22,8 +22,6 @@
 
 #include <config.h>
 
-/* strnlen */
-#define __USE_GNU
 #include <string.h>
 #include <time.h>
 
@@ -34,14 +32,14 @@
 #include <telepathy-glib/errors.h>
 #include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/svc-connection.h>
-#include <telepathy-glib/channel-factory-iface.h>
+#include <telepathy-glib/channel-manager.h>
 
 #define IDLE_DEBUG_FLAG IDLE_DEBUG_CONNECTION
 #include "idle-ctcp.h"
 #include "idle-debug.h"
 #include "idle-handles.h"
-#include "idle-im-factory.h"
-#include "idle-muc-factory.h"
+#include "idle-im-manager.h"
+#include "idle-muc-manager.h"
 #include "idle-parser.h"
 #include "idle-server-connection.h"
 #include "idle-server-connection-iface.h"
@@ -120,6 +118,7 @@ enum {
 	PROP_PORT,
 	PROP_PASSWORD,
 	PROP_REALNAME,
+	PROP_USERNAME,
 	PROP_CHARSET,
 	PROP_QUITMESSAGE,
 	PROP_USE_SSL,
@@ -142,6 +141,7 @@ struct _IdleConnectionPrivate {
 	guint port;
 	char *password;
 	char *realname;
+	char *username;
 	char *charset;
 	char *quit_message;
 	gboolean use_ssl;
@@ -176,7 +176,7 @@ struct _IdleConnectionPrivate {
 #define IDLE_CONNECTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), IDLE_TYPE_CONNECTION, IdleConnectionPrivate))
 
 static void _iface_create_handle_repos(TpBaseConnection *self, TpHandleRepoIface **repos);
-static GPtrArray *_iface_create_channel_factories(TpBaseConnection *self);
+static GPtrArray *_iface_create_channel_managers(TpBaseConnection *self);
 static gchar *_iface_get_unique_connection_name(TpBaseConnection *self);
 static void _iface_disconnected(TpBaseConnection *self);
 static void _iface_shut_down(TpBaseConnection *self);
@@ -243,6 +243,11 @@ static void idle_connection_set_property(GObject *obj, guint prop_id, const GVal
 			priv->realname = g_value_dup_string(value);
 			break;
 
+		case PROP_USERNAME:
+			g_free(priv->username);
+			priv->username = g_value_dup_string(value);
+			break;
+
 		case PROP_CHARSET:
 			g_free(priv->charset);
 			priv->charset = g_value_dup_string(value);
@@ -287,6 +292,10 @@ static void idle_connection_get_property(GObject *obj, guint prop_id, GValue *va
 			g_value_set_string(value, priv->realname);
 			break;
 
+		case PROP_USERNAME:
+			g_value_set_string(value, priv->username);
+			break;
+
 		case PROP_CHARSET:
 			g_value_set_string(value, priv->charset);
 			break;
@@ -342,6 +351,7 @@ static void idle_connection_finalize (GObject *object) {
 	g_free(priv->server);
 	g_free(priv->password);
 	g_free(priv->realname);
+	g_free(priv->username);
 	g_free(priv->charset);
 	g_free(priv->relay_prefix);
 	g_free(priv->quit_message);
@@ -361,6 +371,7 @@ static void idle_connection_class_init(IdleConnectionClass *klass) {
 	static const gchar *interfaces_always_present[] = {
 		TP_IFACE_CONNECTION_INTERFACE_ALIASING,
 		IDLE_IFACE_CONNECTION_INTERFACE_RENAMING,
+		TP_IFACE_CONNECTION_INTERFACE_REQUESTS,
 		NULL};
 
 	g_type_class_add_private(klass, sizeof(IdleConnectionPrivate));
@@ -373,7 +384,8 @@ static void idle_connection_class_init(IdleConnectionClass *klass) {
 
 	parent_class->create_handle_repos = _iface_create_handle_repos;
 	parent_class->get_unique_connection_name = _iface_get_unique_connection_name;
-	parent_class->create_channel_factories = _iface_create_channel_factories;
+	parent_class->create_channel_factories = NULL;
+	parent_class->create_channel_managers = _iface_create_channel_managers;
 	parent_class->connecting = NULL;
 	parent_class->connected = NULL;
 	parent_class->disconnected = _iface_disconnected;
@@ -396,6 +408,9 @@ static void idle_connection_class_init(IdleConnectionClass *klass) {
 	param_spec = g_param_spec_string("realname", "Real name", "The real name of the user connecting to IRC", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB);
 	g_object_class_install_property(object_class, PROP_REALNAME, param_spec);
 
+	param_spec = g_param_spec_string("username", "User name", "The username of the user connecting to IRC", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB);
+	g_object_class_install_property(object_class, PROP_USERNAME, param_spec);
+
 	param_spec = g_param_spec_string("charset", "Character set", "The character set to use to communicate with the outside world", "NULL", G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB | G_PARAM_CONSTRUCT);
 	g_object_class_install_property(object_class, PROP_CHARSET, param_spec);
 
@@ -406,17 +421,17 @@ static void idle_connection_class_init(IdleConnectionClass *klass) {
 	g_object_class_install_property(object_class, PROP_USE_SSL, param_spec);
 }
 
-static GPtrArray *_iface_create_channel_factories(TpBaseConnection *self) {
-	GPtrArray *factories = g_ptr_array_sized_new(1);
-	GObject *factory;
+static GPtrArray *_iface_create_channel_managers(TpBaseConnection *self) {
+	GPtrArray *managers = g_ptr_array_sized_new(1);
+	GObject *manager;
 
-	factory = g_object_new(IDLE_TYPE_IM_FACTORY, "connection", self, NULL);
-	g_ptr_array_add(factories, factory);
+	manager = g_object_new(IDLE_TYPE_IM_MANAGER, "connection", self, NULL);
+	g_ptr_array_add(managers, manager);
 
-	factory = g_object_new(IDLE_TYPE_MUC_FACTORY, "connection", self, NULL);
-	g_ptr_array_add(factories, factory);
+	manager = g_object_new(IDLE_TYPE_MUC_MANAGER, "connection", self, NULL);
+	g_ptr_array_add(managers, manager);
 
-	return factories;
+	return managers;
 }
 
 static void _iface_create_handle_repos(TpBaseConnection *self, TpHandleRepoIface **repos) {
@@ -429,7 +444,7 @@ static void _iface_create_handle_repos(TpBaseConnection *self, TpHandleRepoIface
 static gchar *_iface_get_unique_connection_name(TpBaseConnection *self) {
 	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(self);
 
-	return g_strdup_printf("%s@%s", priv->nickname, priv->server);
+	return g_strdup_printf("%s@%s%p", priv->nickname, priv->server, self);
 }
 
 static gboolean _finish_shutdown_idle_func(gpointer data) {
@@ -506,6 +521,11 @@ static gboolean _iface_start_connecting(TpBaseConnection *self, GError **error)
 				priv->realname = g_strdup(priv->nickname);
 		}
 
+		if (!priv->username || !priv->username[0]) {
+			g_free(priv->username);
+			priv->username = g_strdup(priv->nickname);
+		}
+
 		sconn = IDLE_SERVER_CONNECTION_IFACE(g_object_new(connection_type, "host", priv->server, "port", priv->port, NULL));
 
 		g_signal_connect(sconn, "status-changed", (GCallback)(sconn_status_changed_cb), conn);
@@ -725,7 +745,7 @@ static void _send_with_priority(IdleConnection *conn, const gchar *msg, guint pr
 }
 
 void idle_connection_send(IdleConnection *conn, const gchar *msg) {
-	return _send_with_priority(conn, msg, SERVER_CMD_NORMAL_PRIORITY);
+	_send_with_priority(conn, msg, SERVER_CMD_NORMAL_PRIORITY);
 }
 
 gsize
@@ -766,6 +786,7 @@ static IdleParserHandlerResult _nick_handler(IdleParser *parser, IdleParserMessa
 		return IDLE_PARSER_HANDLER_RESULT_NOT_HANDLED;
 
 	if (old_handle == conn->parent.self_handle) {
+		IDLE_DEBUG("Self renamed: handle was %d, now %d", old_handle, new_handle);
 		tp_base_connection_set_self_handle(TP_BASE_CONNECTION(conn), new_handle);
 	}
 
@@ -864,7 +885,7 @@ static void irc_handshakes(IdleConnection *conn) {
 	g_snprintf(msg, IRC_MSG_MAXLEN + 1, "NICK %s", priv->nickname);
 	idle_connection_send(conn, msg);
 
-	g_snprintf(msg, IRC_MSG_MAXLEN + 1, "USER %s %u * :%s", priv->nickname, 8, priv->realname);
+	g_snprintf(msg, IRC_MSG_MAXLEN + 1, "USER %s %u * :%s", priv->username, 8, priv->realname);
 	idle_connection_send(conn, msg);
 
 	/* gather some information about ourselves */
@@ -874,9 +895,10 @@ static void irc_handshakes(IdleConnection *conn) {
 
 static void send_quit_request(IdleConnection *conn) {
 	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
-	gchar cmd[IRC_MSG_MAXLEN + 1];
+	gchar cmd[IRC_MSG_MAXLEN + 1] = "QUIT";
 
-	g_snprintf(cmd, IRC_MSG_MAXLEN + 1, "QUIT :%s", priv->quit_message);
+	if (priv->quit_message != NULL)
+		g_snprintf(cmd, IRC_MSG_MAXLEN + 1, "QUIT :%s", priv->quit_message);
 
 	_send_with_priority(conn, cmd, SERVER_CMD_MAX_PRIORITY);
 }
