@@ -27,7 +27,7 @@
 G_BEGIN_DECLS
 
 void idle_handle_repos_init(TpHandleRepoIface **handles);
-gboolean idle_nickname_is_valid(const gchar *nickname);
+gboolean idle_nickname_is_valid(const gchar *nickname, gboolean strict_mode);
 
 G_END_DECLS
 
