@@ -40,22 +40,34 @@
 
 static void _channel_iface_init(gpointer, gpointer);
 static void _text_iface_init(gpointer, gpointer);
+static void _destroyable_iface_init(gpointer, gpointer);
 
 G_DEFINE_TYPE_WITH_CODE(IdleIMChannel, idle_im_channel, G_TYPE_OBJECT,
 		G_IMPLEMENT_INTERFACE(TP_TYPE_SVC_CHANNEL, _channel_iface_init);
 		G_IMPLEMENT_INTERFACE(TP_TYPE_SVC_CHANNEL_TYPE_TEXT, _text_iface_init);
-		G_IMPLEMENT_INTERFACE(TP_TYPE_CHANNEL_IFACE, NULL);)
+		G_IMPLEMENT_INTERFACE(TP_TYPE_CHANNEL_IFACE, NULL);
+		G_IMPLEMENT_INTERFACE(TP_TYPE_EXPORTABLE_CHANNEL, NULL);
+		G_IMPLEMENT_INTERFACE(TP_TYPE_SVC_CHANNEL_INTERFACE_DESTROYABLE, _destroyable_iface_init);)
 
 /* property enum */
 enum {
 	PROP_CONNECTION = 1,
 	PROP_OBJECT_PATH,
+	PROP_INTERFACES,
 	PROP_CHANNEL_TYPE,
 	PROP_HANDLE_TYPE,
 	PROP_HANDLE,
+	PROP_TARGET_ID,
+	PROP_REQUESTED,
+	PROP_INITIATOR_HANDLE,
+	PROP_INITIATOR_ID,
+	PROP_CHANNEL_DESTROYED,
+	PROP_CHANNEL_PROPERTIES,
 	LAST_PROPERTY_ENUM
 };
 
+const gchar *im_channel_interfaces[] = {NULL};
+
 /* private structure */
 typedef struct _IdleIMChannelPrivate IdleIMChannelPrivate;
 
@@ -63,6 +75,7 @@ struct _IdleIMChannelPrivate {
 	IdleConnection *connection;
 	gchar *object_path;
 	TpHandle handle;
+	TpHandle initiator;
 
 	gboolean closed;
 
@@ -88,6 +101,7 @@ static GObject *idle_im_channel_constructor(GType type, guint n_props, GObjectCo
 
 	handles = tp_base_connection_get_handles(TP_BASE_CONNECTION(priv->connection), TP_HANDLE_TYPE_CONTACT);
 	tp_handle_ref(handles, priv->handle);
+	tp_handle_ref(handles, priv->initiator);
 	g_assert(tp_handle_is_valid(tp_base_connection_get_handles(TP_BASE_CONNECTION(priv->connection), TP_HANDLE_TYPE_CONTACT), priv->handle, NULL));
 
 	bus = tp_get_bus();
@@ -106,12 +120,15 @@ static GObject *idle_im_channel_constructor(GType type, guint n_props, GObjectCo
 static void idle_im_channel_get_property(GObject *object, guint property_id, GValue *value, GParamSpec *pspec) {
 	IdleIMChannel *chan;
 	IdleIMChannelPrivate *priv;
+	TpHandleRepoIface *handles;
 
 	g_assert(object != NULL);
 	g_assert(IDLE_IS_IM_CHANNEL(object));
 
 	chan = IDLE_IM_CHANNEL(object);
 	priv = IDLE_IM_CHANNEL_GET_PRIVATE(chan);
+	handles = tp_base_connection_get_handles(
+		TP_BASE_CONNECTION(priv->connection), TP_HANDLE_TYPE_CONTACT);
 
 	switch (property_id) {
 		case PROP_CONNECTION:
@@ -122,6 +139,10 @@ static void idle_im_channel_get_property(GObject *object, guint property_id, GVa
 			g_value_set_string(value, priv->object_path);
 			break;
 
+		case PROP_INTERFACES:
+			g_value_set_static_boxed(value, im_channel_interfaces);
+			break;
+
 		case PROP_CHANNEL_TYPE:
 			g_value_set_string(value, TP_IFACE_CHANNEL_TYPE_TEXT);
 			break;
@@ -134,6 +155,46 @@ static void idle_im_channel_get_property(GObject *object, guint property_id, GVa
 			g_value_set_uint(value, priv->handle);
 			break;
 
+		case PROP_TARGET_ID:
+			g_value_set_string(value,
+				tp_handle_inspect(handles, priv->handle));
+			break;
+
+		case PROP_REQUESTED:
+			g_value_set_boolean(value, priv->initiator != priv->handle);
+			break;
+
+		case PROP_INITIATOR_HANDLE:
+			g_value_set_uint(value, priv->initiator);
+			break;
+
+		case PROP_INITIATOR_ID:
+			g_value_set_string(value,
+				tp_handle_inspect(handles, priv->initiator));
+			break;
+
+		case PROP_CHANNEL_DESTROYED:
+			g_value_set_boolean (value, priv->closed);
+			break;
+
+		case PROP_CHANNEL_PROPERTIES:
+		{
+			GHashTable *props =
+				tp_dbus_properties_mixin_make_properties_hash (
+					object,
+					TP_IFACE_CHANNEL, "Interfaces",
+					TP_IFACE_CHANNEL, "ChannelType",
+					TP_IFACE_CHANNEL, "TargetHandleType",
+					TP_IFACE_CHANNEL, "TargetHandle",
+					TP_IFACE_CHANNEL, "TargetID",
+					TP_IFACE_CHANNEL, "InitiatorHandle",
+					TP_IFACE_CHANNEL, "InitiatorID",
+					TP_IFACE_CHANNEL, "Requested",
+					NULL);
+			g_value_take_boxed (value, props);
+			break;
+		}
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
 			break;
@@ -165,6 +226,10 @@ static void idle_im_channel_set_property(GObject *object, guint property_id, con
 			priv->handle = g_value_get_uint(value);
 			break;
 
+		case PROP_INITIATOR_HANDLE:
+			priv->initiator = g_value_get_uint(value);
+			break;
+
 		case PROP_CHANNEL_TYPE:
 		case PROP_HANDLE_TYPE:
 			/* writeable in the interface, but setting them makes
@@ -195,18 +260,69 @@ static void idle_im_channel_class_init (IdleIMChannelClass *idle_im_channel_clas
 	g_object_class_override_property(object_class, PROP_CHANNEL_TYPE, "channel-type");
 	g_object_class_override_property(object_class, PROP_HANDLE_TYPE, "handle-type");
 	g_object_class_override_property(object_class, PROP_HANDLE, "handle");
+	g_object_class_override_property(object_class, PROP_CHANNEL_DESTROYED, "channel-destroyed");
+	g_object_class_override_property(object_class, PROP_CHANNEL_PROPERTIES, "channel-properties");
 
 	param_spec = g_param_spec_object("connection", "IdleConnection object",
-			"The IdleConnection object that owns this "
-			"IMChannel object.",
-			IDLE_TYPE_CONNECTION,
-			G_PARAM_CONSTRUCT_ONLY |
-			G_PARAM_READWRITE |
-			G_PARAM_STATIC_NICK |
-			G_PARAM_STATIC_BLURB);
+									 "The IdleConnection object that owns this "
+									 "IMChannel object.",
+									 IDLE_TYPE_CONNECTION,
+									 G_PARAM_CONSTRUCT_ONLY |
+									 G_PARAM_READWRITE |
+									 G_PARAM_STATIC_NICK |
+									 G_PARAM_STATIC_BLURB);
 	g_object_class_install_property(object_class, PROP_CONNECTION, param_spec);
 
+	param_spec = g_param_spec_boxed ("interfaces", "Extra D-Bus interfaces",
+		"Interfaces implemented by this object besides Channel",
+		G_TYPE_STRV, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+	g_object_class_install_property (object_class, PROP_INTERFACES, param_spec);
+
+	param_spec = g_param_spec_string ("target-id", "Contact's identifier",
+		"The name of the person we're speaking to",
+		NULL, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+	g_object_class_install_property (object_class, PROP_TARGET_ID, param_spec);
+
+	param_spec = g_param_spec_boolean ("requested", "Requested?",
+		"True if this channel was requested by the local user",
+		FALSE,
+		G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+	g_object_class_install_property (object_class, PROP_REQUESTED, param_spec);
+
+	param_spec = g_param_spec_uint ("initiator-handle", "Initiator's handle",
+		"The contact who initiated the channel",
+		0, G_MAXUINT32, 0,
+		G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+	g_object_class_install_property (object_class, PROP_INITIATOR_HANDLE, param_spec);
+
+	param_spec = g_param_spec_string ("initiator-id", "Initiator's bare JID",
+		"The string obtained by inspecting the initiator-handle",
+		NULL, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+	g_object_class_install_property (object_class, PROP_INITIATOR_ID, param_spec);
+
+	static TpDBusPropertiesMixinPropImpl channel_props[] = {
+		{ "Interfaces", "interfaces", NULL },
+		{ "ChannelType", "channel-type", NULL },
+		{ "TargetHandleType", "handle-type", NULL },
+		{ "TargetHandle", "handle", NULL },
+		{ "TargetID", "target-id", NULL },
+		{ "InitiatorHandle", "initiator-handle", NULL },
+		{ "InitiatorID", "initiator-id", NULL },
+		{ "Requested", "requested", NULL },
+		{ NULL }
+	};
+	static TpDBusPropertiesMixinIfaceImpl prop_interfaces[] = {
+		{ TP_IFACE_CHANNEL,
+			tp_dbus_properties_mixin_getter_gobject_properties,
+			NULL,
+			channel_props,
+		},
+		{ NULL }
+	};
+
+	idle_im_channel_class->dbus_props_class.interfaces = prop_interfaces;
 	tp_text_mixin_class_init(object_class, G_STRUCT_OFFSET(IdleIMChannelClass, text_class));
+	tp_dbus_properties_mixin_class_init(object_class, G_STRUCT_OFFSET(IdleIMChannelClass, dbus_props_class));
 }
 
 void idle_im_channel_dispose (GObject *object) {
@@ -234,6 +350,7 @@ void idle_im_channel_finalize (GObject *object) {
 
 	handles = tp_base_connection_get_handles(TP_BASE_CONNECTION(priv->connection), TP_HANDLE_TYPE_CONTACT);
 	tp_handle_unref(handles, priv->handle);
+	tp_handle_unref(handles, priv->initiator);
 
 	if (priv->object_path)
 		g_free(priv->object_path);
@@ -272,10 +389,35 @@ static void idle_im_channel_close (TpSvcChannel *iface, DBusGMethodInvocation *c
 	g_assert(IDLE_IS_IM_CHANNEL(obj));
 
 	priv = IDLE_IM_CHANNEL_GET_PRIVATE(obj);
-	priv->closed = TRUE;
 
 	IDLE_DEBUG("called on %p", obj);
 
+	/* The IM manager will resurrect the channel if we have pending
+	 * messages. When we're resurrected, we want the initiator
+	 * to be the contact who sent us those messages, if it isn't already */
+	if (tp_text_mixin_has_pending_messages((GObject *)obj, NULL)) {
+		IDLE_DEBUG("Not really closing, I still have pending messages");
+
+		if (priv->initiator != priv->handle) {
+			TpHandleRepoIface *contact_repo =
+				tp_base_connection_get_handles(
+					(TpBaseConnection *) priv->connection,
+					TP_HANDLE_TYPE_CONTACT);
+
+			g_assert(priv->initiator != 0);
+			g_assert(priv->handle != 0);
+
+			tp_handle_unref(contact_repo, priv->initiator);
+			priv->initiator = priv->handle;
+			tp_handle_ref(contact_repo, priv->initiator);
+		}
+
+		tp_text_mixin_set_rescued((GObject *) obj);
+	} else {
+		IDLE_DEBUG ("Actually closing, I have no pending messages");
+		priv->closed = TRUE;
+	}
+
 	tp_svc_channel_emit_closed(iface);
 
 	tp_svc_channel_return_from_close(context);
@@ -337,9 +479,7 @@ static void idle_im_channel_get_handle (TpSvcChannel *iface, DBusGMethodInvocati
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
 static void idle_im_channel_get_interfaces(TpSvcChannel *iface, DBusGMethodInvocation *context) {
-	const gchar *interfaces[] = {NULL};
-
-	tp_svc_channel_return_from_get_interfaces(context, interfaces);
+	tp_svc_channel_return_from_get_interfaces(context, im_channel_interfaces);
 }
 
 /**
@@ -373,6 +513,18 @@ static void idle_im_channel_send(TpSvcChannelTypeText *iface, guint type, const
 	idle_text_send((GObject *)(obj), type, recipient, text, priv->connection, context);
 }
 
+static void idle_im_channel_destroy(TpSvcChannelInterfaceDestroyable *iface, DBusGMethodInvocation *context) {
+	IdleIMChannel *obj = (IdleIMChannel *)(iface);
+	IdleIMChannelPrivate *priv = IDLE_IM_CHANNEL_GET_PRIVATE(obj);
+
+	IDLE_DEBUG ("called on %p with %spending messages", obj,
+		tp_text_mixin_has_pending_messages((GObject *)obj, NULL) ? "" : "no ");
+
+	priv->closed = TRUE;
+	tp_svc_channel_emit_closed(iface);
+	tp_svc_channel_interface_destroyable_return_from_destroy(context);
+}
+
 static void _channel_iface_init(gpointer g_iface, gpointer iface_data) {
 	TpSvcChannelClass *klass = (TpSvcChannelClass *)g_iface;
 
@@ -395,3 +547,9 @@ static void _text_iface_init(gpointer g_iface, gpointer iface_data) {
 #undef IMPLEMENT
 }
 
+static void _destroyable_iface_init(gpointer klass, gpointer iface_data) {
+#define IMPLEMENT(x) tp_svc_channel_interface_destroyable_implement_##x (\
+		klass, idle_im_channel_##x)
+	IMPLEMENT (destroy);
+#undef IMPLEMENT
+}
